package cerebral;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.dnd.DragSource;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.InputEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSlider;
import javax.swing.ScrollPaneConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.MouseInputAdapter;

import publicdomain.ScrollableFlowPanel;

/* This code was originally written with the panel at the top of the display.
 * Given that horizontal resolution is much greater than vertical resolution, 
 * I finally realised that the panel makes a lot more sense down the left side 
 * of the screen.
 * So you may have to do some mental gymnastics, such as replacing left with up
 * and right with down in the variable and function names.
 * */

public class SmallMultipleContainer extends JPanel implements
		PropertyChangeListener {

	private static final long serialVersionUID = 1L;

	private List views = new ArrayList();

	private ViewAndTitle select1;

	private ViewAndTitle select2;

	private ViewCoordinator viewCoordinator;

	private int dragX, dragY;

	private ViewAndTitleContainer dragSource;

	private ViewAndTitleContainer dragDest;

	private Cerebral cerebral;

	private JSlider viewSizeSlider;

	private ScrollableFlowPanel viewsPanel = new ScrollableFlowPanel();

	private int viewSize = 1;

	private JScrollPane scrollViews;

	private ViewAndTitleContainer compare1;

	private Color unselected;

	public SmallMultipleContainer(Cerebral cerebral,
			final ViewCoordinator viewCoordinator) {

		this.viewCoordinator = viewCoordinator;
		viewCoordinator.setSmallMultipleContainer(this);
		this.cerebral = cerebral;
		// ControlPanel.instance().addPropertyChangeListener(this);

		viewsPanel.setLayout(new FlowLayout(FlowLayout.CENTER));
		scrollViews = new JScrollPane(viewsPanel);
		// scrollViews.getViewport().setBackground(SystemColor.control);
		scrollViews
				.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		scrollViews.addComponentListener(new ComponentAdapter() {
			public void componentResized(ComponentEvent e) {
				updateSmallMultipleSize();
			}
		});

		setLayout(new BorderLayout());
		add(scrollViews, BorderLayout.CENTER);
		viewSizeSlider = new JSlider(JSlider.HORIZONTAL, 1, 6, 6);
		add(viewSizeSlider, BorderLayout.PAGE_END);

		viewSizeSlider.setPaintLabels(false);
		viewSizeSlider.setPaintTicks(false);
		viewSizeSlider.setSnapToTicks(true);
		viewSizeSlider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				if (!source.getValueIsAdjusting()) {
					viewSize = 6 - source.getValue() + 1;
					updateSmallMultipleSize();
				}
			}
		});

		dragSource = null;
		dragX = 0;
		dragY = 0;

		addMouseListener(new MouseAdapter() {
			public void mouseExited(MouseEvent e) {
				setCursor(Cursor.getDefaultCursor());
			}

			public void mouseEntered(MouseEvent e) {
				if (dragSource != null) {
					setCursor(DragSource.DefaultMoveDrop);
				}
			}
		});

	}

	public void viewSelected(ViewAndTitle viewAndTitle, boolean shiftDown) {
		if (!shiftDown) {
			if (viewAndTitle == select1 && select2 == null) {
				deselect(select1);
				select1 = null;
				viewCoordinator.setComparison(null, null);
			} else {
				deselect(select1);
				deselect(select2);
				select1 = viewAndTitle;
				select2 = null;
				select(select1, true);
				viewCoordinator.setComparison(select1.view, null);
			}
		} else {
			if (viewAndTitle == select1) {
				select1 = null;
				deselect(viewAndTitle);
				viewCoordinator.setComparison(null, null);
			} else {
				if (select1 == null) {
					select1 = viewAndTitle;
					select(select1, true);
				} else if (select2 == null) {
					select2 = viewAndTitle;
					select(select2, false);
				} else {
					deselect(select2);
					select2 = viewAndTitle;
					select(select2, false);
				}
				if (select1 != null && select2 != null) {
					viewCoordinator.setComparison(select1.view, select2.view);
				} else if (select1 != null && select2 == null) {
					viewCoordinator.setComparison(select1.view, null);
				} else
					viewCoordinator.setComparison(null, null);

			}
		}

	}

	private void select(ViewAndTitle viewAndTitle, boolean primary) {
		JLabel title = viewAndTitle.title;

		if (primary) {
			title.setForeground(Color.white);
			title.setOpaque(true);
			title.setBackground(Color.blue);
		} else {
			title.setForeground(Color.black);
			title.setOpaque(true);
			title.setBackground(Color.blue);
		}
		title.repaint();

	}

	private void deselect(ViewAndTitle viewAndTitle) {
		if (viewAndTitle != null) {
			JLabel title = viewAndTitle.title;
			title.setForeground(Color.black);
			title.setBackground(unselected);
			// title.setBackground(Color.black);

			title.repaint();
		}

	}

	private void shiftRight(int start, int end) {
		for (int i = end; i > start; i--) {
			ViewAndTitleContainer leftNeighbour = (ViewAndTitleContainer) views
					.get(i - 1);
			ViewAndTitleContainer view = (ViewAndTitleContainer) views.get(i);
			view.changeContents(leftNeighbour.viewAndTitle);
		}
	}

	private void shiftLeft(int start, int end) {
		for (int i = start; i < end; i++) {
			ViewAndTitleContainer rightNeighbour = (ViewAndTitleContainer) views
					.get(i + 1);
			ViewAndTitleContainer view = (ViewAndTitleContainer) views.get(i);
			view.changeContents(rightNeighbour.viewAndTitle);

		}
	}

	private void setView(int location, ViewAndTitle viewAndTitle) {
		ViewAndTitleContainer target = (ViewAndTitleContainer) views
				.get(location);
		target.changeContents(viewAndTitle);
	}

	public ViewAndTitleContainer findView(String title) {
		for (int i = 0; i < views.size(); i++) {
			ViewAndTitleContainer check = (ViewAndTitleContainer) views.get(i);
			if (check.viewAndTitle.title.getText().compareTo(title) == 0) {
				return check;
			}
		}
		return null;
	}

	public int findLocation(ViewAndTitleContainer x) {
		for (int i = 0; i < views.size(); i++) {
			ViewAndTitleContainer check = (ViewAndTitleContainer) views.get(i);
			if (check == x) {
				return i;
			}
		}
		return -1;
	}

	private class DragController extends MouseInputAdapter {
		private String id;

		public DragController(JLabel id) {
			this.id = id.getText();
		}

		public void mouseDragged(MouseEvent e) {
			if (e.getX() - dragX > 5 || e.getY() - dragY > 5) {
				dragSource = findView(id);
				setCursor(DragSource.DefaultMoveDrop);
			}
		}

		public void mouseClicked(MouseEvent e) {

			if (e.getButton() == MouseEvent.BUTTON1) {
				if (compare1 != null) {
					viewSelected(findView(id).viewAndTitle, true);
					compare1 = null;
				} else {
					viewSelected(
							findView(id).viewAndTitle,
							(e.getModifiers() & InputEvent.SHIFT_MASK) == InputEvent.SHIFT_MASK);
				}
				dragX = e.getX();
				dragY = e.getY();

			}
		}

		public void mouseEntered(MouseEvent e) {
			dragDest = findView(id);
		}

		public void mouseReleased(MouseEvent e) {
			if (dragDest != null && dragSource != null) {
				if (dragSource != dragDest) {
					ViewAndTitle saveViewAndTitle = dragSource.viewAndTitle;
					int sourceLoc = findLocation(dragSource);
					int destLoc = findLocation(dragDest);

					if (sourceLoc < destLoc) {
						shiftLeft(sourceLoc, destLoc);
					} else {
						shiftRight(destLoc, sourceLoc);
					}

					setView(destLoc, saveViewAndTitle);
					viewCoordinator.viewMoved(sourceLoc, destLoc);

				}

			}
			dragSource = null;
			setCursor(Cursor.getDefaultCursor());
		}

	}

	public void addView(SmallMultipleView view) {

		ViewAndTitle viewAndTitle = new ViewAndTitle(view);
		ViewAndTitleContainer viewAndTitleContainer = new ViewAndTitleContainer(
				viewAndTitle, this);

		viewsPanel.add(viewAndTitleContainer, new Integer(views.size())
				.toString());
		views.add(viewAndTitleContainer);
		viewAndTitleContainer.addPropertyChangeListener("compare", this);
		DragController dragController = new DragController(viewAndTitle.title);
		unselected = viewAndTitle.title.getBackground();
		viewAndTitleContainer.viewAndTitle.title
				.addMouseListener(dragController);
		viewAndTitleContainer.viewAndTitle.title
				.addMouseMotionListener(dragController);
		viewAndTitleContainer.viewAndTitle.view
				.addMouseListener(dragController);
		viewAndTitleContainer.viewAndTitle.view
				.addMouseMotionListener(dragController);

		// viewAndTitleContainer.addMouseListener(dragController);
		// viewAndTitleContainer.addMouseMotionListener(dragController);

	}

	public class ViewAndTitle {
		public JLabel title;

		public SmallMultipleView view;

		public ViewAndTitle(SmallMultipleView smallMultipleView) {
			title = new JLabel(smallMultipleView.getConditonName());
			this.view = smallMultipleView;
		}
	}

	public class ViewAndTitleContainer extends JPanel {
		private static final long serialVersionUID = 1L;

		ViewAndTitle viewAndTitle;
		SmallMultipleContainer smc;

		public ViewAndTitleContainer(final ViewAndTitle viewAndTitle,
				SmallMultipleContainer smc) {
			this.viewAndTitle = viewAndTitle;
			this.smc = smc;

			setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
			setLayout(new GridBagLayout());
			populate();
		}

		public void changeContents(ViewAndTitle newViewAndTitle) {
			removeAll();
			this.viewAndTitle = newViewAndTitle;
			populate();
			validate();
			repaint();

		}

		private void populate() {
			GridBagConstraints c = new GridBagConstraints();
			c.gridx = 0;
			c.gridy = 0;
			c.fill = GridBagConstraints.HORIZONTAL;
			c.weighty = 0.0;
			c.weightx = 1.0;
			add(viewAndTitle.title, c);

			JButton compare = new JButton();
			compare.setIcon(new ImageIcon(getClass().getResource(
					"/cerebral/icons/compare1.png")));
			compare.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					firePropertyChange("compare", viewAndTitle.title.getText(),
							null);
				}
			});
			compare.setToolTipText("Compare against another condition");
			c.gridx = 1;
			c.fill = GridBagConstraints.NONE;
			c.anchor = GridBagConstraints.LINE_END;
			add(compare, c);

			c.fill = GridBagConstraints.BOTH;
			c.gridx = 0;
			c.gridwidth = 2;
			c.gridy = 1;
			c.weighty = 1.0;
			add(viewAndTitle.view, c);
		}

	}

	public void moveView(String axis, int sourceLoc, int destLoc) {
		ViewAndTitleContainer aContainer = findView(axis);
		// Ugh. This null check is a hack. Why should it be null? Investigate
		// further
		if (aContainer != null) {
			ViewAndTitle saveViewAndTitle = aContainer.viewAndTitle;

			if (sourceLoc < destLoc) {
				shiftLeft(sourceLoc, destLoc);
			} else {
				shiftRight(destLoc, sourceLoc);
			}

			setView(destLoc, saveViewAndTitle);
		}

	}

	public void reset() {
		viewsPanel.removeAll();
		views.clear();
		select1 = null;
		select2 = null;
	}

	private void updateSmallMultipleSize() {

		int newWidth = (scrollViews.getViewport().getWidth() - (viewSize * 16))
				/ viewSize;

		// int newWidth = (int) (cerebral.getWidth() * viewSize);
		if (newWidth < 2) {
			newWidth = 2;
		}
		Rectangle2D nodebounds = cerebral.getVisualization().getBounds(
				Cerebral.NODES);
		double aspectRatio = nodebounds.getHeight() / nodebounds.getWidth();

		int newHeight = (int) (aspectRatio * newWidth);
		for (int i = 0; i < views.size(); i++) {
			ViewAndTitleContainer vatc = (ViewAndTitleContainer) views.get(i);
			vatc.viewAndTitle.view.setSize(newWidth, newHeight);
		}

		scrollViews.getViewport().validate();
		if (getParent() != null) {
			getParent().validate();
		}

		viewCoordinator.fitViewToBounds(1000);

	}

	public void propertyChange(PropertyChangeEvent e) {
		if (e.getPropertyName().compareTo("compare") == 0) {
			if (compare1 == null) {
				compare1 = findView((String) e.getOldValue());
				if (compare1.viewAndTitle != select1) {
					viewSelected(compare1.viewAndTitle, false);
				}
			} else {
				ViewAndTitleContainer compare2 = findView((String) e
						.getOldValue());
				if (compare2 == compare1) {
					deselect(compare1.viewAndTitle);
					viewSelected(compare1.viewAndTitle, false);
				} else {
					select(compare2.viewAndTitle, false);
					viewSelected(compare2.viewAndTitle, true);
				}
				compare1 = null;
			}
		}
	}

}
