package cerebral.colors;

import java.awt.BasicStroke;
import java.awt.Color;
import java.util.Map;

import prefuse.Visualization;
import prefuse.action.ActionList;
import prefuse.action.RepaintAction;
import prefuse.action.assignment.ColorAction;
import prefuse.action.assignment.StrokeAction;
import prefuse.data.Table;
import prefuse.data.column.ColumnMetadata;
import prefuse.data.tuple.TupleSet;
import prefuse.util.ColorLib;
import prefuse.util.StrokeLib;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;
import cerebral.Cerebral;
import cerebral.ControlPanel;
import cytoscape.CyNetwork;
import cytoscape.Cytoscape;
import cytoscape.data.CyAttributesUtils;
import cytoscape.visual.LineType;

public class ExpressionColorManager {

	private CyNetwork cyNetwork;

	private ActionList fillColor;

	private ExpressionValue EV = new ExpressionValue();

	private ExpressionValue CV = new ExpressionValue();

	private Map expressionValues;

	private Map expressionSignificance;

	private Map compareValue;

	private Map compareSignificance;

	public ExpressionColorManager(CyNetwork cyNetwork, String condition) {
		this.cyNetwork = cyNetwork;

		expressionValues = CyAttributesUtils.getAttribute(condition, Cytoscape
				.getNodeAttributes());

		expressionSignificance = getSignificanceMap(condition);

	}

	public ActionList getFillColor() {
		FillNodeAction fillNodeAction = new FillNodeAction(Cerebral.NODES);
		BorderAction borderAction = new BorderAction(Cerebral.NODES);

		ActionList fill = new ActionList();
		fill.add(fillNodeAction);
		fill.add(borderAction);

		return fill;
	}

	public ActionList getComparisonFill(String conditionName) {
		ComparisonFill fillNodeAction = new ComparisonFill(Cerebral.NODES,
				conditionName);
		ComparisonBorderAction borderAction = new ComparisonBorderAction(
				Cerebral.NODES);

		ActionList fill = new ActionList();
		fill.add(fillNodeAction);
		fill.add(borderAction);

		return fill;
	}

	public void createColorRules(Visualization vis, Table edges) {
		FillNodeAction fillNodeAction = new FillNodeAction(Cerebral.NODES);
		BorderAction borderAction = new BorderAction(Cerebral.NODES);

		fillColor = new ActionList();
		fillColor.add(fillNodeAction);

		// create an action list containing all color assignments
		ActionList color = new ActionList();
		color.add(fillColor);
		color.add(borderAction);
		color.add(new BorderStrokeAction(Cerebral.NODES));
		color.add(new RepaintAction());
		color.add(new MyStrokeAction(Cerebral.EDGES));
		color.add(new EdgeFadeColorAction(Cerebral.EDGES, edges));
		vis.putAction("color", color);

	}

	public class BorderAction extends ColorAction {
		private BorderAction(String group) {
			super(group, VisualItem.STROKECOLOR);
		}

		public int getColor(VisualItem item) {
			if (!isForeground(item, m_vis)) {
				return ColorManager.increasedContrast(ColorLib.color(m_vis
						.getDisplay(0).getBackground()));
			}
			if (item.getBoolean(VisualItem.HOVER)) {
				return ColorManager.Highlight1;
			} else {
				return ColorLib.gray(0);
			}
		}
	}

	public class MyStrokeAction extends StrokeAction {

		MyStrokeAction(String group) {
			super(group);
		}

		public BasicStroke getStroke(VisualItem item) {

			TupleSet selectedItems = m_vis.getFocusGroup(Cerebral.SELECTED);
			boolean fullColor = false;
			if (selectedItems.containsTuple(item)) {
				fullColor = true;
			}
			float width = 1.0f;

			LineType lt = (LineType) item.get(Cerebral.LINE_TYPE);
			BasicStroke basicStroke;
			if (lt != null) {
				basicStroke = (BasicStroke) lt.getStroke();
			} else {
				basicStroke = StrokeLib.getStroke(width, BasicStroke.CAP_ROUND,
						BasicStroke.JOIN_ROUND);
			}
			width = basicStroke.getLineWidth();
			if (!fullColor) {
				return StrokeLib.getStroke(
						(float) (basicStroke.getLineWidth()),
						BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND,
						basicStroke.getMiterLimit(),
						basicStroke.getDashArray(), basicStroke.getDashPhase());
			} else {

				if (item.isInGroup(Cerebral.SINGLE_HIGHLIGHT)) {
					return StrokeLib
							.getStroke((float) (1.8 * basicStroke
									.getLineWidth()), BasicStroke.CAP_ROUND,
									BasicStroke.JOIN_ROUND, basicStroke
											.getMiterLimit(), basicStroke
											.getDashArray(), basicStroke
											.getDashPhase());
				} else if (item.getBoolean(VisualItem.HOVER)) {
					return StrokeLib
							.getStroke((float) (1.5 * basicStroke
									.getLineWidth()), BasicStroke.CAP_ROUND,
									BasicStroke.JOIN_ROUND, basicStroke
											.getMiterLimit(), basicStroke
											.getDashArray(), basicStroke
											.getDashPhase());
				} else if (item.getBoolean(VisualItem.HIGHLIGHT)) {
					return StrokeLib
							.getStroke((float) (1.5 * basicStroke
									.getLineWidth()), BasicStroke.CAP_ROUND,
									BasicStroke.JOIN_ROUND, basicStroke
											.getMiterLimit(), basicStroke
											.getDashArray(), basicStroke
											.getDashPhase());
				} else if (item.isInGroup(Cerebral.NEIGHB_SELECT)) {
					return StrokeLib
							.getStroke((float) (1.5 * basicStroke
									.getLineWidth()), BasicStroke.CAP_ROUND,
									BasicStroke.JOIN_ROUND, basicStroke
											.getMiterLimit(), basicStroke
											.getDashArray(), basicStroke
											.getDashPhase());
				} else {
					return StrokeLib
							.getStroke((float) (basicStroke.getLineWidth()),
									BasicStroke.CAP_ROUND,
									BasicStroke.JOIN_ROUND, basicStroke
											.getMiterLimit(), basicStroke
											.getDashArray(), basicStroke
											.getDashPhase());
				}
			}

		}
	}

	public class BorderStrokeAction extends StrokeAction {
		private BorderStrokeAction(String group) {
			super(group);
		}

		public BasicStroke getStroke(VisualItem item) {
			if (item.getBoolean(VisualItem.HOVER)) {
				return StrokeLib.getStroke(3.0f);
			} else {
				return StrokeLib.getStroke(1.0f);
			}
		}
	}

	public class FillNodeAction extends ColorAction {
		private FillNodeAction(String group) {
			super(group, VisualItem.FILLCOLOR);
		}

		public int getColor(VisualItem item) {
			if (isForeground(item, m_vis)) {
				return ControlPanel.instance().getExpressionColor().getColor(
						EV.value);
			} else {

				return ColorLib.rgba(0, 0, 0, 0);
			}
		}
	}

	private boolean isForeground(VisualItem item, Visualization vis) {
		NodeItem node = (NodeItem) item;

		TupleSet selectedItems = vis.getFocusGroup(Cerebral.SELECTED);
		TupleSet hoverItems = vis.getFocusGroup(Cerebral.NEIGHB_SELECT);
		boolean isForeground = false;
		if (selectedItems.containsTuple(node) || hoverItems.containsTuple(node)) {
			isForeground = true;
		}
		if (node.isHover())
			isForeground = true;

		if (node.getBoolean(Cerebral.IGNORE)) {
			isForeground = false;
		}

		String id = cyNetwork.getNode(node.getInt(Cerebral.CYTOSCAPE_ID))
				.getIdentifier();

		getExpressionValue(EV, id, expressionValues, expressionSignificance);
		if (!EV.hasValue) {
			isForeground = false;
		} else if (Math.abs(EV.value) < ControlPanel.instance()
				.getFoldChangeCutoff()) {
			isForeground = false;
		}
		if (!EV.hasSignificance) {
			isForeground = false;
		}
		return isForeground;
	}

	public class EdgeFadeColorAction extends ColorAction {

		private Table edges;

		public EdgeFadeColorAction(String group, Table edges) {
			super(group, VisualItem.STROKECOLOR);
			this.edges = edges;
		}

		public int getColor(VisualItem item) {
			EdgeItem edge = (EdgeItem) item;
			TupleSet selectedItems = m_vis.getFocusGroup(Cerebral.SELECTED);
			TupleSet hoverItems = m_vis.getFocusGroup(Cerebral.NEIGHB_SELECT);
			boolean fullColor = false;
			if (selectedItems.containsTuple(edge)
					|| hoverItems.containsTuple(edge)) {
				fullColor = true;
			}
			if (edge.isHover())
				fullColor = true;
			if (!fullColor) {

				return ColorLib.rgba(0, 0, 0, 45);

			} else {

				if (item.isInGroup(Cerebral.SINGLE_HIGHLIGHT))
					return ColorManager.Highlight2;
				if (item.getBoolean(VisualItem.HOVER))
					return ColorManager.Highlight1;
				if (item.getBoolean(VisualItem.HIGHLIGHT))
					return ColorManager.Highlight1;
				if (item.isInGroup(Cerebral.NEIGHB_SELECT))
					return ColorManager.Highlight1;

				ColumnMetadata edgeLengthMetadata = edges
						.getMetadata(Cerebral.EDGE_LENGTH);
				int maxRow = edgeLengthMetadata.getMaximumRow();
				float maxLength = edges.getInt(maxRow, Cerebral.EDGE_LENGTH);

				Color c = (Color) edge.get(Cerebral.FILL_COLOR);
				if (c == null) {
					return ColorManager.increasedContrast(ColorLib.color(m_vis
							.getDisplay(0).getBackground()));
				}

				int edgeLength = edge.getInt(Cerebral.EDGE_LENGTH);
				double normalizedLength = 1 - (edgeLength / maxLength);
				double alphaFactor = normalizedLength * normalizedLength
						* normalizedLength * normalizedLength;

				return ColorManager.changeSaturationTo(c,
						(float) ((100 + 155 * alphaFactor) / 255.0f));
			}
		}
	}

	public class ComparisonFill extends ColorAction {

		private ComparisonFill(String group, String compare) {
			super(group, VisualItem.FILLCOLOR);
			compareValue = CyAttributesUtils.getAttribute(compare, Cytoscape
					.getNodeAttributes());
			compareSignificance = getSignificanceMap(compare);

		}

		public int getColor(VisualItem item) {
			if (compareIsForground(item, m_vis)) {
				double difference = 0.0;
				if (ControlPanel.instance().isRatioData()) {
					if (CV.hasSignificance && EV.hasSignificance) {
						difference = (CV.oneAdjustedData() - EV
								.oneAdjustedData());
						// / Math.abs(EV.value);
						difference += difference > 0 ? 1 : -1;
					} else if (CV.hasSignificance) {
						difference = -CV.value;
					} else if (EV.hasSignificance) {
						difference = EV.value;
					}
				} else {
					if (CV.hasSignificance && EV.hasSignificance) {
						difference = CV.value - EV.value;
					} else if (CV.hasSignificance) {
						difference = -CV.value;
					} else if (EV.hasSignificance) {
						difference = EV.value;
					}
				}
				return ControlPanel.instance().getComparisonColor().getColor(
						difference);

			} else {
				return ColorLib.rgba(0, 0, 0, 0);
			}
		}
	}

	public class ComparisonBorderAction extends ColorAction {

		private ComparisonBorderAction(String group) {
			super(group, VisualItem.STROKECOLOR);
		}

		public int getColor(VisualItem item) {

			if (!compareIsForground(item, m_vis)) {
				return ColorManager.increasedContrast(ColorLib.color(m_vis
						.getDisplay(0).getBackground()));
			}

			if (item.getBoolean(VisualItem.HOVER)) {
				return ColorManager.Highlight1;
			} else {
				return ColorLib.gray(0);
			}
		}
	}

	private boolean compareIsForground(VisualItem item, Visualization vis) {
		NodeItem node = (NodeItem) item;

		TupleSet selectedItems = vis.getFocusGroup(Cerebral.SELECTED);
		TupleSet hoverItems = vis.getFocusGroup(Cerebral.NEIGHB_SELECT);
		boolean isForeground = false;
		if (selectedItems.containsTuple(node) || hoverItems.containsTuple(node)) {
			isForeground = true;
		}
		if (node.isHover())
			isForeground = true;

		String id = cyNetwork.getNode(node.getInt(Cerebral.CYTOSCAPE_ID))
				.getIdentifier();

		if (isForeground) {
			getExpressionValue(EV, id, expressionValues, expressionSignificance);
			getExpressionValue(CV, id, compareValue, compareSignificance);

			if (!EV.hasValue || !CV.hasValue) {
				// Don't show if we don't have a value for one of the
				// conditions
				isForeground = false;
			}
			if (!(EV.hasSignificance || CV.hasSignificance)) {
				// show foreground if either value is significant
				// but not if both values are insignificant
				isForeground = false;
			}

			double difference = 0.0;
			if (ControlPanel.instance().isRatioData()) {
				if (CV.hasSignificance && EV.hasSignificance) {
					difference = (CV.oneAdjustedData() - EV.oneAdjustedData());
					// / Math.abs(EV.value);
					difference += difference > 0 ? 1 : -1;
				} else if (CV.hasSignificance) {
					difference = -CV.value;
				} else if (EV.hasSignificance) {
					difference = EV.value;
				}
			} else {
				if (CV.hasSignificance && EV.hasSignificance) {
					difference = CV.value - EV.value;
				} else if (CV.hasSignificance) {
					difference = -CV.value;
				} else if (EV.hasSignificance) {
					difference = EV.value;
				}
			}

			if (Math.abs(difference) < ControlPanel.instance()
					.getFoldChangeCutoff()) {
				isForeground = false;
			}
		}
		return isForeground;

	}

	private class ExpressionValue {
		public boolean hasValue;

		public double value;

		public boolean hasSignificance;

		public double oneAdjustedData() {
			if (value > 0.0)
				return value - 1.0;
			else
				return value + 1.0;
		}

	};

	private void getExpressionValue(ExpressionValue ev, String cyNodeID,
			Map values, Map significance) {

		ev.hasValue = false;

		if (values == null) {
			ev.hasValue = false;
		} else {

			Object attribVal = values.get(cyNodeID);
			if (attribVal instanceof Double) {
				ev.value = ((Double) attribVal).doubleValue();
				ev.hasValue = true;
			} else if (attribVal instanceof Integer) {
				ev.value = ((Integer) attribVal).intValue();
				ev.hasValue = true;
			} else if (attribVal instanceof String) {
				try {
					ev.hasValue = true;
					ev.value = Double.parseDouble((String) attribVal);
				} catch (NumberFormatException e) {
					ev.hasValue = false;
				}
			} else {
				ev.hasValue = false;
			}
		}

		if (significance == null) {
			ev.hasSignificance = true;
		} else {
			Object val = significance.get(cyNodeID);
			if (val instanceof Double) {
				ev.hasSignificance = ((Double) val).doubleValue() < ControlPanel
						.instance().getSigCutoff();
			} else if (val instanceof Integer) {
				ev.hasSignificance = ((Integer) val).intValue() < ControlPanel
						.instance().getSigCutoff();

			} else if (val instanceof String) {
				try {
					ev.hasSignificance = Double.parseDouble((String) val) < ControlPanel
							.instance().getSigCutoff();

				} catch (NumberFormatException e) {
					ev.hasSignificance = false;
				}
			} else {
				ev.hasSignificance = false;
			}
		}

	}

	private Map getSignificanceMap(String condition) {

		Map sig = CyAttributesUtils.getAttribute(condition + "_sig", Cytoscape
				.getNodeAttributes());
		if (sig != null) {
			return sig;
		}
		sig = CyAttributesUtils.getAttribute(condition + "sig", Cytoscape
				.getNodeAttributes());
		if (sig != null) {
			return sig;
		}

		sig = CyAttributesUtils.getAttribute(condition + "_pvals", Cytoscape
				.getNodeAttributes());
		if (sig != null) {
			return sig;
		}
		sig = CyAttributesUtils.getAttribute(condition + "pvals", Cytoscape
				.getNodeAttributes());
		if (sig != null) {
			return sig;
		}
		if (condition.endsWith("exp")) {
			String testName = condition.substring(0, condition.length() - 3)
					+ "sig";
			sig = CyAttributesUtils.getAttribute(testName, Cytoscape
					.getNodeAttributes());
			return sig;

		}
		return null;

	}
}
