package cerebral.parallelCoordinates.render;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import cerebral.parallelCoordinates.CoordinateSorter;
import cerebral.parallelCoordinates.ParallelCoordinates;

import prefuse.Display;
import prefuse.data.tuple.TupleSet;
import prefuse.render.Renderer;
import prefuse.util.ColorLib;
import prefuse.util.FontLib;
import prefuse.visual.VisualItem;

public class ClusterGlyphRenderer implements Renderer {

    private Line2D line = new Line2D.Double();

    private Rectangle2D rect = new Rectangle2D.Double();

    private Display d;

    private Display clusterD;

    private CoordinateSorter coordinateSorter = new CoordinateSorter();

    private int[] coordinateOrder;

    VisualItem[] sorter;

    private double zoom;
    private double pan;

    public ClusterGlyphRenderer(Display d, Display clusterD, int dimensions, double scale) {
        this.d = d;
        this.clusterD = clusterD;
        coordinateOrder = new int[dimensions];
        for (int i = 0; i < dimensions; i++) {
            coordinateOrder[i] = i;
        }
        sorter = new VisualItem[dimensions];
        this.zoom = scale;
        pan = 20;

    }

    public boolean locatePoint(Point2D p, VisualItem item) {
        return item.getBounds().contains(p);
    }

    public void render(Graphics2D g, VisualItem item) {
        List values = (List) item.get("dataPoints");

        // Frame
        rect.setRect(item.getX(), item.getY(), 40, 40);
        g.setColor(Color.black);
        g.draw(rect);

        // Label
        int count = item.getInt("count");
        String countLabel = Integer.toString(count);
        g.setFont(FontLib.getFont("Verdana", 12.0));
        g.drawString(countLabel, (int) item.getX() + 2, (int) (item.getY()
                + 40 - 2));

        double xInterval = 40 / (values.size() - 1.0);
        double xPos = item.getX();
        double prevPosition = item.getY()
                + getPosition(item, (Double) values.get(coordinateOrder[0]));
        g.setStroke(item.getStroke());
        for (int i = 1; i < values.size(); i++) {
            double yPos = item.getY()
                    + getPosition(item, (Double) values.get(coordinateOrder[i]));
            line.setLine(xPos, prevPosition, xPos + xInterval, yPos);
            g.setColor(ColorLib.getColor(item.getFillColor()));
            g.draw(line);
            prevPosition = yPos;
            xPos += xInterval;
        }
    }

    private double getPosition(VisualItem item, Double val) {
        return (-val.doubleValue()) / zoom + pan;
    }

    public void setBounds(VisualItem item) {
        item
                .setBounds(item.getX(), item.getY(), 40,40);

    }

    public void validate() {

        TupleSet coordinates = d.getVisualization().getVisualGroup(
                ParallelCoordinates.COORDINATES);

        int i = 0;
        Iterator coordsIt = coordinates.tuples();
        while (coordsIt.hasNext()) {
            VisualItem coordinate = (VisualItem) coordsIt.next();
            sorter[i++] = coordinate;
        }

        Arrays.sort(sorter, coordinateSorter);
        for (int j = 0; j < coordinateOrder.length; j++) {
            coordinateOrder[j] = sorter[j].getRow();
        }

        Iterator dataIt = clusterD.getVisualization()
                .getVisualGroup("clusters").tuples();
        while (dataIt.hasNext()) {
            VisualItem item = (VisualItem) dataIt.next();
            item.setValidated(false);
        }
        clusterD.getVisualization().repaint();

    }

    public void zoomToFit(double topVal, double bottomVal) {
        double range = Math.abs(topVal - bottomVal);
        zoom =range / 40;
        pan = bottomVal / zoom;
       
    }   
}