package cerebral.parallelCoordinates.render;

import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import cerebral.parallelCoordinates.CoordinateSorter;
import cerebral.parallelCoordinates.ParallelCoordinates;
import cerebral.parallelCoordinates.YDisplay;

import prefuse.Display;
import prefuse.data.tuple.TupleSet;
import prefuse.render.Renderer;
import prefuse.util.ColorLib;
import prefuse.visual.DecoratorItem;
import prefuse.visual.VisualItem;

public class DataRenderer implements Renderer, PropertyChangeListener {

    private int[] coordinateOrder;

    private double[] coordinateLocation;

    private Display d;

    private CoordinateSorter coordinateSorter = new CoordinateSorter();

    private VisualItem[] sorter;

    private Line2D line = new Line2D.Double();

    private Rectangle2D coordinateBounds = new Rectangle2D.Double();

    private YDisplay yDisplay;

    public DataRenderer(Display d, YDisplay yDisplay, int dataDimensions) {

        coordinateLocation = new double[dataDimensions];
        coordinateOrder = new int[dataDimensions];
        sorter = new VisualItem[dataDimensions];

        this.d = d;
        this.yDisplay = yDisplay;
        coordinateBounds.setRect(-1000000, 10, 2000000, 100);
        yDisplay.addPropertyChangeListener(this);

    }

    public void validate() {
        TupleSet coordinates = d.getVisualization().getVisualGroup(
                ParallelCoordinates.COORDINATES);

        if (coordinates != null) {
            int i = 0;
            Iterator coordsIt = coordinates.tuples();
            while (coordsIt.hasNext() && i < sorter.length) {
                VisualItem coordinate = (VisualItem) coordsIt.next();
                sorter[i++] = coordinate;
            }

            Arrays.sort(sorter, coordinateSorter);
            for (int j = 0; j < coordinateOrder.length; j++) {
                coordinateOrder[j] = sorter[j].getRow();
                coordinateLocation[j] = sorter[j].getX();
            }

            Iterator dataIt = d.getVisualization().getVisualGroup(
                    ParallelCoordinates.DATA).tuples();
            while (dataIt.hasNext()) {
                VisualItem item = (VisualItem) dataIt.next();
                item.setValidated(false);
            }
        }

    }

    public boolean locatePoint(Point2D p, VisualItem item) {
        double prevPosition;
        List values = (List) item.get(ParallelCoordinates.VALUES);
        prevPosition = yDisplay.getPosition((Double) values
                .get(coordinateOrder[0]));

        for (int i = 1; i < values.size(); i++) {
            double yPos = yDisplay.getPosition((Double) values
                    .get(coordinateOrder[i]));
            line.setLine(coordinateLocation[i - 1], prevPosition,
                    coordinateLocation[i], yPos);
            if (line.intersects(p.getX() - 1, p.getY() - 1, 2, 2))
                return true;
            prevPosition = yPos;
        }
        return false;
    }

    public void render(Graphics2D g, VisualItem item) {
        double prevPosition;
        List values = (List) item.get(ParallelCoordinates.VALUES);
        prevPosition = yDisplay.getPosition((Double) values
                .get(coordinateOrder[0]));
        g.setColor(ColorLib.getColor(item.getFillColor()));

        g.setClip(coordinateBounds);
        g.setStroke(item.getStroke());
        for (int i = 1; i < values.size(); i++) {
            double yPos = yDisplay.getPosition((Double) values
                    .get(coordinateOrder[i]));
            line.setLine(coordinateLocation[i - 1], prevPosition,
                    coordinateLocation[i], yPos);
            g.draw(line);
            prevPosition = yPos;
        }
        g.setClip(null);
    }

    public void setBounds(VisualItem item) {
        /*
         * int id = item.getRow(); double yMin = 100000000.0; double yMax =
         * -100000000.0;
         * 
         * for (int i = 0; i < data[id].length; i++) { double yPos =
         * getPosition(data[id][i]); if (yPos < yMin) { yMin = yPos; } if (yPos >
         * yMax) { yMax = yPos; } } item.setBounds(-1000000, yMin, 2000000, yMax -
         * yMin);
         */
        item.setBounds(coordinateBounds.getX(), coordinateBounds.getY(),
                coordinateBounds.getWidth(), coordinateBounds.getHeight());
    }

    public boolean itemIntersectsRect(VisualItem item, Rectangle2D rect) {
        double prevPosition;
        List values = (List) item.get(ParallelCoordinates.VALUES);
        prevPosition = yDisplay.getPosition((Double) values
                .get(coordinateOrder[0]));

        for (int i = 1; i < values.size(); i++) {
            double yPos = yDisplay.getPosition((Double) values
                    .get(coordinateOrder[i]));
            line.setLine(coordinateLocation[i - 1], prevPosition,
                    coordinateLocation[i], yPos);
            if (line.intersects(rect))
                return true;
            prevPosition = yPos;
        }
        return false;
    }

    public void setBounds(double leftMargin, int top, double width, double h) {
        coordinateBounds.setRect(0, top, width*2, h);
    }

    public void propertyChange(PropertyChangeEvent evt) {
        validate();
        d.repaint();

        TupleSet bottomLabels = d.getVisualization().getVisualGroup(
                ParallelCoordinates.FILTER_BOTTOM_LABELS);
        if (bottomLabels != null) {

            Iterator it = bottomLabels.tuples();
            while (it.hasNext()) {
                DecoratorItem label = (DecoratorItem) it.next();
                VisualItem coordinate = label.getDecoratedItem();
                label.setY(yDisplay.getPosition(coordinate
                        .getDouble(ParallelCoordinates.BOTTOM)));
            }
        }

        TupleSet topLabels = d.getVisualization().getVisualGroup(
                ParallelCoordinates.FILTER_TOP_LABELS);
        if (topLabels != null) {
            Iterator it = topLabels.tuples();
            while (it.hasNext()) {
                DecoratorItem label = (DecoratorItem) it.next();
                VisualItem coordinate = label.getDecoratedItem();
                label.setY(yDisplay.getPosition(coordinate
                        .getDouble(ParallelCoordinates.TOP)));
            }
        }
    }

}
