package cerebral.parallelCoordinates.render;

import java.awt.Graphics2D;
import java.awt.geom.Rectangle2D;
import java.text.NumberFormat;

import cerebral.parallelCoordinates.ParallelCoordinates;
import cerebral.parallelCoordinates.YDisplay;

import prefuse.Constants;
import prefuse.render.LabelRenderer;
import prefuse.visual.VisualItem;

public class IntervalLabelRenderer extends LabelRenderer {

    private YDisplay yDisplay;
    private String field;
    private NumberFormat nf = NumberFormat.getInstance();
    private Rectangle2D clip = new Rectangle2D.Double();

    
    public IntervalLabelRenderer(YDisplay yDisplay, String numericField){
        this.yDisplay = yDisplay;
        clip.setRect(-1000000, 10, 2000000, 100);
        field = numericField; 
        nf.setMaximumFractionDigits(2);
        setHorizontalAlignment(Constants.LEFT);
    }
    
    public void render(Graphics2D g, VisualItem item) {
        double val = item.getDouble(field);
        item.setY(yDisplay.getPosition(val));
        VisualItem coordinate = (VisualItem) item.get(ParallelCoordinates.COORDINATE);
        item.setX(coordinate.getX());
        g.setClip(clip);
        super.render(g, item);
        g.setClip(null);
        
    }
    
    protected String getText(VisualItem item) {
        double val = item.getDouble(field);
        return  nf.format(val);
    }
    
    public void setBounds(int top, double height) {
        clip.setRect(-1000000, top, 20000000, height);
    }
    
}
