package cerebral.render;

import java.awt.Shape;
import java.awt.geom.GeneralPath;
import java.awt.geom.RoundRectangle2D;

import prefuse.Constants;
import prefuse.render.ShapeRenderer;
import prefuse.visual.VisualItem;

//Extend the ShapeRender with a few extra shapes needed by Cytoscape
public class ExtendedShapeRenderer extends ShapeRenderer {
	  public static final int SHAPE_ROUNDED_RECTANGLE = Constants.SHAPE_COUNT + 1;
	  public static final int SHAPE_PARALLELOGRAM = Constants.SHAPE_COUNT + 2;
	  public static final int SHAPE_OCTAGON = Constants.SHAPE_COUNT + 3;
		 
	  private RoundRectangle2D roundedRect = new RoundRectangle2D.Double();
	  private GeneralPath path = new GeneralPath();
	  
	  public Shape getFinalShape(VisualItem item){
		  double x = item.getEndX();
	        if ( Double.isNaN(x) || Double.isInfinite(x) )
	            x = 0;
	        double y = item.getEndY();
	        if ( Double.isNaN(y) || Double.isInfinite(y) )
	            y = 0;
	        return createShape(item,  x, y);
	  }
	  
	  private Shape createShape(VisualItem item, double x, double y){
		  int stype = item.getShape();
	        double width = getBaseSize()*item.getSize();
	        
	        // Center the shape around the specified x and y
	        if ( width > 1 ) {
	            x = x-width/2;
	            y = y-width/2;
	        }
	        
	        switch ( stype ) {
	        case Constants.SHAPE_NONE:
	            return null;
	        case Constants.SHAPE_RECTANGLE:
	            return rectangle(x, y, width, width);
	        case SHAPE_ROUNDED_RECTANGLE:
	        	return roundedRectangle(x,y,width,width);
	        case SHAPE_PARALLELOGRAM:
	        	return parallelogram((float)x,(float)y,(float)width,(float)width);
	        case SHAPE_OCTAGON:
	        	return octagon ((float)x,(float)y,(float)width);
	        case Constants.SHAPE_ELLIPSE:
	            return ellipse(x, y, width, width);
	        case Constants.SHAPE_TRIANGLE_UP:
	            return triangle_up((float)x, (float)y, (float)width);
	        case Constants.SHAPE_TRIANGLE_DOWN:
	            return triangle_down((float)x, (float)y, (float)width);
	        case Constants.SHAPE_TRIANGLE_LEFT:
	            return triangle_left((float)x, (float)y, (float)width);
	        case Constants.SHAPE_TRIANGLE_RIGHT:
	            return triangle_right((float)x, (float)y, (float)width);
	        case Constants.SHAPE_CROSS:
	            return cross((float)x, (float)y, (float)width);
	        case Constants.SHAPE_STAR:
	            return star((float)x, (float)y, (float)width);
	        case Constants.SHAPE_HEXAGON:
	            return hexagon((float)x, (float)y, (float)width);
	        case Constants.SHAPE_DIAMOND:
	            return diamond((float)x, (float)y, (float)width);
	        default:
	            throw new IllegalStateException("Unknown shape type: "+stype);
	        }

	  }
	  protected Shape getRawShape(VisualItem item) {
	      double x = item.getX();
	        if ( Double.isNaN(x) || Double.isInfinite(x) )
	            x = 0;
	        double y = item.getY();
	        if ( Double.isNaN(y) || Double.isInfinite(y) )
	            y = 0;
	        return createShape(item,  x, y);
	  
	  }

	  private Shape octagon(float x, float y,  float height) {
	        float width = height;  
	        float width3 = height/3;
	        float height3 = height/3;
	        
	        path.reset();
	        path.moveTo(x,                   y + height3);
	        path.lineTo(x + width3,          y);
	        path.lineTo(x + width3 + width3, y);
	        path.lineTo(x + width,           y + height3);
	        path.lineTo(x + width,           y + height3 + height3);
	        path.lineTo(x + width3 + width3, y + height);
	        path.lineTo(x + width3,          y + height);
	        path.lineTo(x,                   y + height3 + height3);
	        path.closePath();
	        
	        return path;
	}

	public Shape roundedRectangle(double x, double y, double width, double height) {
	        roundedRect.setRoundRect(x,y,width,height, width/4, height/4);
	        return roundedRect;
	    }
	  public Shape parallelogram(float x, float y, float width, float height) {
		    width = height*2/3;
	        path.reset();
	        path.moveTo(x ,y);
	        path.lineTo(x + width/2 , y +height);
	        path.lineTo(x + width + width/2, y+height);
	        path.lineTo(x + width, y);
	        
	        path.closePath();
	        return path;
	    }

}
